(function() {

	//configure the Alkane Code TinyMCE plugin
	
    var pm = tinymce.PluginManager;
    var du = tinymce.dom.DOMUtils;   

    var getLanguages = function(editor) {
        return editor.settings.alkanecode_languages;
    };

    var getDialogMinWidth = function(editor) {
        return Math.min(du.DOM.getViewPort().w, editor.getParam('alkanecode_dialog_width', 800));
    };

    var getDialogMinHeight = function(editor) {
        return Math.min(du.DOM.getViewPort().w, editor.getParam('alkanecode_dialog_height', 650));
    };

    var Settings = {
        getLanguages: getLanguages,
        getDialogMinWidth: getDialogMinWidth,
        getDialogMinHeight: getDialogMinHeight
    };

    //if element is a <pre> or <code> with a language- class
    function isAlkaneCode(elm) {
        var rtn = elm && (elm.nodeName === 'PRE' || elm.nodeName === 'CODE');
        return rtn;
    }

    function trimArg(predicateFn) {
        return function(arg1, arg2) {
            return predicateFn(arg2);
        };
    }

    var Utils = {
        isAlkaneCode: isAlkaneCode,
        trimArg: trimArg
    };

    var getSelectedAlkaneCode = function(editor) {
        var node = editor.selection.getNode();
        if (Utils.isAlkaneCode(node)) {
            return node;
        }
        return null;
    };

    var insertAlkaneCode = function(editor, language, inline, code) {
						
        editor.undoManager.transact(function() {
		
		
            var node = getSelectedAlkaneCode(editor);
            code = du.DOM.encode(code);			
			
            var nodeclass = "";
            var classNoLanguage = "";

            if (node) {				
                //get any existing classes
                nodeclass = node.getAttribute('class');				
				
				if (nodeclass != null && nodeclass != "") {
					classNoLanguage = nodeclass.replace(/\blang(?:uage)?-(?!\*)(.[^"]+)\b/, "");
					classNoLanguage = classNoLanguage.trim();
                }
				
                //if node exists remove it and we'll create with proper semantics
                if ((node.parentNode.nodeName.toLowerCase()) == "pre") {
                    editor.dom.remove(node.parentNode);
                } else {
                    editor.dom.remove(node);
                }				
            }

            var newElement;
			var classstring = "";			
			var spacer = "";
		
			if (classNoLanguage != "" || language != "") {
				//we have at least one class to add
				classstring = ' class="';
				
				if (classNoLanguage != "") {
					classstring += classNoLanguage;
					spacer = " ";
				}
				if (language != "") {
					classstring += spacer + "language-" + language;
				}
				classstring += '"';							
			}			
						
            //if new piece of code
            if (inline) {
				//remove line breaks				
				code = code.replace(/(\r\n|\n|\r)/gm," ");
				editor.insertContent('<code ' + classstring + '>' + code + '</code>');	
					
            } else {
				editor.insertContent('<pre><code ' + classstring + '>' + code + '</code></pre>');  
		
            }

            editor.selection.select(newElement);

        });
    };

    var getCurrentCode = function(editor) {
        var node = getSelectedAlkaneCode(editor);
        if (node) {
            return node.textContent;
        }
        return '';
    };

    var getCurrentInline = function(editor) {
        var node = getSelectedAlkaneCode(editor);
        if (node) {
			//if we either click on a <pre>, or the parent node is a <pre>, it's a block of code
            if ((node.parentNode.nodeName.toLowerCase()) == "pre" || (node.nodeName.toLowerCase()) == "pre") {
                return false;
            } else {
				//it's inline code
				return true;
			}
        }
		//return block code by default (we dont lose formatting then by default)
        return false;
    };

    var AlkaneCode = {
        getSelectedAlkaneCode: getSelectedAlkaneCode,
        insertAlkaneCode: insertAlkaneCode,
        getCurrentInline: getCurrentInline,
        getCurrentCode: getCurrentCode
    };

    var getLanguages$1 = function(editor) {       
        var customLanguages = Settings.getLanguages(editor);
        return customLanguages;
    };

    var getCurrentLanguage = function(editor) {
        var matches;
        var node = AlkaneCode.getSelectedAlkaneCode(editor);
		
        if (node) {
		
			if (node.nodeName.toLowerCase() == "pre")
			{
				var codenode = node.firstChild;				
				if (codenode) 
				{
					node = codenode;
				}				
			}
		
            matches = node.className.match(/\blang(?:uage)?-(?!\*)(.[^"]+)\b/);		
            return matches ? matches[1] : '';
        }
        return '';
    };

    var Languages = {
        getLanguages: getLanguages$1,
        getCurrentLanguage: getCurrentLanguage
    };


    pm.add('alkanecode', function(editor, url) {

        var Dialog = {
            open: function(editor) {
                var minWidth = Settings.getDialogMinWidth(editor);
                var minHeight = Settings.getDialogMinHeight(editor);
                var currentLanguage = Languages.getCurrentLanguage(editor);
                var currentLanguages = Languages.getLanguages(editor);
                var currentInline = AlkaneCode.getCurrentInline(editor);
                var currentCode = AlkaneCode.getCurrentCode(editor);

                editor.windowManager.open({
                    title: 'Insert/Edit Alkane Code',
                    minWidth: minWidth,
                    minHeight: minHeight,
                    layout: 'flex',
                    direction: 'column',
                    align: 'stretch',
                    body: [{
                            type: 'listbox',
                            name: 'language',
                            label: 'Language',
                            maxWidth: 200,
                            value: currentLanguage,
                            values: currentLanguages
                        },
                        {
                            type: 'checkbox',
                            name: 'inline',
                            label: 'Inline',
                            checked: currentInline
                        },				
						{
							type: 'label', 
							text: '(Selecting \'Inline\' will remove line breaks after clicking \'OK\'.)'},						
                        {
                            type: 'textbox',
                            name: 'code',
                            multiline: true,
                            spellcheck: false,
                            ariaLabel: 'Code view',
                            flex: 1,
                            style: 'direction: ltr; text-align: left',
                            classes: 'monospace',
                            value: currentCode,
                            autofocus: true,
							onKeyDown: function( e ) {								
								
								
								var codeeditor = e.target;
								//support tabs
								var keyCode = e.keyCode || e.which;

								if (keyCode == 9) {
									e.preventDefault();
									var start = codeeditor.selectionStart;
									var end = codeeditor.selectionEnd;
									var val = codeeditor.value;
									var selected = val.substring(start, end);
									var re, count;

									if(e.shiftKey) {
										re = /^\t/gm;
										if (selected.length > 0) {
											var mtc = selected.match(re);
											if (mtc != null) {
											  count = -mtc.length;
											  codeeditor.value = val.substring(0, start) + selected.replace(re, '') + val.substring(end);
											}
										 }
									} else {
										re = /^/gm;
										var mtc = selected.match(re);
										if (mtc != null) {
											count = mtc.length;
											codeeditor.value = val.substring(0, start) + selected.replace(re, '\t') + val.substring(end);
										}
									}

									if(start === end) {
										codeeditor.selectionStart = end + count;
									} else {
										codeeditor.selectionStart = start;
									}

									codeeditor.selectionEnd = end + count;
								}
								

								return true;
							}
                        }
                    ],
                    onSubmit: function(e) {
                        AlkaneCode.insertAlkaneCode(editor, e.data.language, e.data.inline, e.data.code);
                    }
                });
            }
        };

        var $ = editor.$;

        editor.on('PreProcess', function(e) {
           
        });


        editor.on('SetContent', function() {
          
        });

        editor.addCommand('alkanecode-command', function() {

            editor.formatter.register('codelanguageformat', {
                inline: 'code'
            });

            var node = editor.selection.getNode();
            if (editor.selection.isCollapsed() || Utils.isAlkaneCode(node)) {
                Dialog.open(editor);
            } else {
                editor.formatter.toggle('codelanguageformat');
            }
        });

		//handle double clicks on <pre> and <code> tags
        editor.on('dblclick', function(ev) {
            //fix to ensure selected
            editor.selection.select(ev.target);
            if (Utils.isAlkaneCode(ev.target)) {
                Dialog.open(editor);
            }
        });


        //add Button to Visual Editor Toolbar
        editor.addButton('alkanecode', {
            title: 'Insert Button Link',
            image: url + '/../img/icon.svg',
            cmd: 'alkanecode-command',
        });
    });
})();